; ##############################################################################
; ##############################################################################
; ##############################################################################
; WDTCSR  Watchdog Timer Control Register										
; WDIF WDIE WDP3 WDCE WDE WDP2 WDP1 WDP0										
; WDP3 WDP2 WDP1 WDP0 Time														
;	0 	0 	0 	0 	2K 	(2048) 	cycles 16 ms									
;	0 	0 	0 	1 	4K 	(4096) 	cycles 32 ms									
;	0 	0 	1 	0 	8K 	(8192) 	cycles 64 ms									
;	0 	0 	1 	1 	16K (16384) cycles 0.125 s									
;	0 	1 	0 	0 	32K (32768) cycles 0.25 s									
; 	0 	1 	0 	1 	64K (65536) cycles 0.5 s									
; *	0 	1 	1 	0 	128K (131072)cycles 1.0 s									
;	0 	1 	1 	1 	256K (262144)cycles 2.0 s									
;	1 	0 	0 	0 	512K (524288)cycles 4.0 s									
;	1 	0 	0 	1 	1024K(1048576)cycles 8.0 s									
; Wenn WDIE=1 dann wird zuerst die WDR-ISR aufgerufen 							
; und dann das System neu gestartet												
WDR_INITIALISIERUNG:
	WDR
	LDS TEMP, WDTCSR
	ori TEMP, (1<<WDCE) | (1<<WDE)
	STS WDTCSR, TEMP
	ldi temp, (1<<WDE)| (1<<WDIE)|(0<<WDP3)|(1<<WDP2)|(1<<WDP1)|(0<<WDP0)
	STS WDTCSR, TEMP
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; MCUSR  MCU Status Register													
; 7,6,5,JTRF,WDRF,BORF,EXTRF,PORF												
; NEUSTART ber WDR, wenn BORF,EXTRF,PORF gestzt ist							
NEUSTART_MIT_WDR:
	in temp, MCUSR
	andi temp,0b00000111
	tst temp
	brne NEUSTART_MIT_WDR_FEHLER
	ret
NEUSTART_MIT_WDR_FEHLER:
	out MCUSR,NULL
	rjmp NEUSTART_MIT_WDR_FEHLER
; ##############################################################################
; ##############################################################################
; ##############################################################################
SRAM_CLEAR:
	ldi ZL, LOW (SRAM_START)
	ldi ZH, HIGH(SRAM_START)
	ldi temp,0
SRAM_CLEAR_w:
	ST z+,temp
	cpi ZH, HIGH(RAMEND-2)	; Vergleich?		
	brlo SRAM_CLEAR_w		; kleiner => SPRUNG	
	cpi ZL, LOW(RAMEND-2)	; Vergleich?		
	brlo SRAM_CLEAR_w		; kleiner => SPRUNG	
	breq SRAM_CLEAR_w		; gleich  => SPRUNG	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; PORT A+B+C+D (alles EINGNGE+PULL-UP aus)										
PORT_CLEAR:
	out DDRA, NULL
	out PORTA,NULL
	out DDRB, NULL
	out PORTB,NULL
	out DDRC, NULL
	out PORTC,NULL
	out DDRD, NULL
	out PORTD,NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
PROGRAMM_CHECK:
	; NULL		
	tst NULL
	brne PROGRAMM_CHECK_FEHLER_SCHWER
	; EINS		
	mov temp,EINS
	cpi temp,1
	brne PROGRAMM_CHECK_FEHLER_SCHWER
	; SRAM CHECK
	LDS temp,(adr_SRAM_CHECK)
	cpi temp,'S'
	brne PROGRAMM_CHECK_FEHLER_SCHWER
	; FARBE		
	LDS temp,(adr_GRAFIC_FARBE)
	andi temp,0b11110000
	tst temp
	brne PROGRAMM_CHECK_ERROR
	ret
;-------------------------------------------------------------------------------
PROGRAMM_CHECK_ERROR:
	STS(adr_ERROR_WERT),temp
	ldi temp,(ERROR_FARBE)
	STS(adr_ERROR_CODE),temp
ret
;-------------------------------------------------------------------------------
PROGRAMM_CHECK_FEHLER_SCHWER:
	CLI											; INTERRUPTS sperren	
	sbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR)		; GRN aus	
	sbi (LED_GELB_PORT),(LED_GELB_PIN_NR)		; GELB aus  
	call wait_100ms
	cbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  an	
	call wait_100ms
	sbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  aus	
	rjmp PROGRAMM_CHECK_FEHLER_SCHWER
; ##############################################################################
; ##############################################################################
; ##############################################################################
LED_INITIALISIERUNG:
	; auf AUSGANG								
	sbi (LED_GRUEN_DDR),(LED_GRUEN_PIN_NR)			; GRN		
	sbi (LED_GELB_DDR),(LED_GELB_PIN_NR)			; GELB		
	sbi (LED_ROT_DDR),(LED_ROT_PIN_NR)				; ROT		

	; LED an									
	cbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR)			; GRN an   	
	cbi (LED_GELB_PORT),(LED_GELB_PIN_NR)			; GELB an		
	cbi (LED_ROT_PORT),(LED_ROT_PIN_NR)				; ROT  an		
	call wait_100ms
	; LED aus									
	sbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR)			; GRN aus	
	sbi (LED_GELB_PORT),(LED_GELB_PIN_NR)			; GELB aus	
	sbi (LED_ROT_PORT),(LED_ROT_PIN_NR)				; ROT  aus	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
IMPULS_ZAEHLER_A:
	LDS temp,(adr_IMPULS_AKTIV_A)
	tst temp
	brne IMPULS_ZAEHLER_A_RUN
	ret
IMPULS_ZAEHLER_A_RUN:
	STS(adr_IMPULS_AKTIV_A),NULL
	cbi (LED_GELB_PORT),(LED_GELB_PIN_NR)			; GELB an		
	STS(adr_FREIGABE_DIAGRAMM_ZEICHNEN),EINS
	LDS temp,(adr_IMPULSE_A)
	cpi temp,255									; berlauf ?	
	brne IMPULS_ZAEHLER_A_w
	ret
IMPULS_ZAEHLER_A_w:
	inc temp
	STS(adr_IMPULSE_A),temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
IMPULS_ZAEHLER_B:
	LDS temp,(adr_IMPULS_AKTIV_B)
	tst temp
	brne IMPULS_ZAEHLER_B_RUN
	ret
IMPULS_ZAEHLER_B_RUN:
	STS(adr_IMPULS_AKTIV_B),NULL
	cbi (LED_GELB_PORT),(LED_GELB_PIN_NR)			; GELB an		
	STS(adr_FREIGABE_DIAGRAMM_ZEICHNEN),EINS
	LDS temp,(adr_IMPULSE_B)
	cpi temp,255									; berlauf ?	
	brne IMPULS_ZAEHLER_B_w
	ret
IMPULS_ZAEHLER_B_w:
	inc temp
	STS(adr_IMPULSE_B),temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
PROGRAMM_STOP:
	WDR
rjmp PROGRAMM_STOP
; ##############################################################################
; ##############################################################################
; ##############################################################################

